"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.LocalOrdererManager = void 0;
const server_memory_orderer_1 = require("@fluidframework/server-memory-orderer");
const server_services_client_1 = require("@fluidframework/server-services-client");
const server_services_core_1 = require("@fluidframework/server-services-core");
/**
 * @internal
 */
class LocalOrdererManager {
    constructor(storage, databaseManager, createHistorian, logger, serviceConfiguration, pubsub, documentRepository, checkpointRepository) {
        this.storage = storage;
        this.databaseManager = databaseManager;
        this.createHistorian = createHistorian;
        this.logger = logger;
        this.serviceConfiguration = serviceConfiguration;
        this.pubsub = pubsub;
        this.documentRepository = documentRepository;
        this.checkpointRepository = checkpointRepository;
        /**
         * Map of "tenantId/documentId" to the orderer for that document.
         */
        this.ordererMap = new Map();
    }
    /**
     * Closes all local orderers
     */
    async close() {
        await Promise.all(Array.from(this.ordererMap.values()).map(async (orderer) => (await orderer).close()));
        this.ordererMap.clear();
    }
    /**
     * Returns true if there are any received ops that are not yet ordered.
     */
    async hasPendingWork() {
        return Promise.all(this.ordererMap.values()).then((orderers) => {
            for (const orderer of orderers) {
                // We know that it ia LocalOrderer, break the abstraction
                if (orderer.hasPendingWork()) {
                    return true;
                }
            }
            return false;
        });
    }
    async getOrderer(tenantId, documentId) {
        const key = `${tenantId}/${documentId}`;
        let orderer = this.ordererMap.get(key);
        if (orderer === undefined) {
            orderer = this.createLocalOrderer(tenantId, documentId);
            this.ordererMap.set(key, orderer);
        }
        return orderer;
    }
    async createLocalOrderer(tenantId, documentId) {
        const historian = await this.createHistorian(tenantId);
        const gitManager = new server_services_client_1.GitManager(historian);
        const documentRepository = this.documentRepository ??
            new server_services_core_1.MongoDocumentRepository(await this.databaseManager.getDocumentCollection());
        const deliCheckpointRepository = this.checkpointRepository ??
            new server_services_core_1.MongoCheckpointRepository(await this.databaseManager.getCheckpointCollection(), "deli");
        const scribeCheckpointRepository = this.checkpointRepository ??
            new server_services_core_1.MongoCheckpointRepository(await this.databaseManager.getCheckpointCollection(), "scribe");
        const deliCheckpointService = new server_services_core_1.CheckpointService(deliCheckpointRepository, documentRepository, false);
        const scribeCheckpointService = new server_services_core_1.CheckpointService(scribeCheckpointRepository, documentRepository, false);
        const orderer = await server_memory_orderer_1.LocalOrderer.load(this.storage, this.databaseManager, tenantId, documentId, this.logger, documentRepository, deliCheckpointRepository, scribeCheckpointRepository, deliCheckpointService, scribeCheckpointService, gitManager, undefined /* ILocalOrdererSetup */, this.pubsub, undefined /* broadcasterContext */, undefined /* scriptoriumContext */, undefined /* scribeContext */, undefined /* deliContext */, undefined /* moiraContext */, this.serviceConfiguration);
        const lambdas = [
            orderer.broadcasterLambda,
            orderer.deliLambda,
            orderer.scribeLambda,
            orderer.scriptoriumLambda,
        ];
        await Promise.all(lambdas.map(async (lambda) => {
            if (lambda === undefined) {
                throw new Error("We expect lambdas to be defined by now.");
            }
            if (lambda.state === "created") {
                return new Promise((resolve) => lambda.once("started", () => resolve()));
            }
        }));
        return orderer;
    }
}
exports.LocalOrdererManager = LocalOrdererManager;
//# sourceMappingURL=localOrdererManager.js.map